@@ -20,7 +20,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#ident "$Id: insmod.c,v 1.4 1997/12/01 00:46:05 rth Exp $"
+#ident "$Id: insmod.c,v 1.8 1998/01/23 09:33:28 rth Exp $"
 
 #include <sys/types.h>
 #include <stdlib.h>
@@ -88,6 +88,8 @@ search_module_path(char *base)
     "/lib/modules/%s/cdrom:"
     "/lib/modules/%s/ipv4:"
     "/lib/modules/%s/ipv6:"
+    "/lib/modules/%s/sound:"
+    "/lib/modules/%s/fc4:"
     "/lib/modules/%s/misc:"
     "/lib/modules/default/fs:"
     "/lib/modules/default/net:"
@@ -96,6 +98,8 @@ search_module_path(char *base)
     "/lib/modules/default/cdrom:"
     "/lib/modules/default/ipv4:"
     "/lib/modules/default/ipv6:"
+    "/lib/modules/default/sound:"
+    "/lib/modules/default/fc4:"
     "/lib/modules/default/misc:"
     "/lib/modules/fs:"
     "/lib/modules/net:"
@@ -104,6 +108,8 @@ search_module_path(char *base)
     "/lib/modules/cdrom:"
     "/lib/modules/ipv4:"
     "/lib/modules/ipv6:"
+    "/lib/modules/sound:"
+    "/lib/modules/fc4:"
     "/lib/modules/misc";
 
   char *path, *p, *filename;
@@ -216,7 +222,7 @@ add_symbols_from(struct obj_file *f, int idx,
       sym = obj_find_symbol(f, (char *)s->name);
       if (sym)
 	{
-	  sym = obj_add_symbol(f, (char *)s->name,
+	  sym = obj_add_symbol(f, (char *)s->name, -1,
 			       ELFW(ST_INFO)(STB_GLOBAL, STT_NOTYPE),
 			       idx, s->value, 0);
 
@@ -512,7 +518,7 @@ old_create_mod_use_count(struct obj_file *f)
   sec = obj_create_alloced_section_first(f, ".moduse", sizeof(long),
 					 sizeof(long));
 
-  obj_add_symbol(f, "mod_use_count_", ELFW(ST_INFO)(STB_LOCAL, STT_OBJECT),
+  obj_add_symbol(f, "mod_use_count_", -1, ELFW(ST_INFO)(STB_LOCAL, STT_OBJECT),
 		 sec->idx, 0, sizeof(long));
 
   return 1;
@@ -1157,7 +1163,7 @@ new_create_this_module(struct obj_file *f, const char *m_name)
 					 sizeof(struct new_module));
   memset(sec->contents, 0, sizeof(struct new_module));
 
-  obj_add_symbol(f, "__this_module", ELFW(ST_INFO)(STB_LOCAL, STT_OBJECT),
+  obj_add_symbol(f, "__this_module", -1, ELFW(ST_INFO)(STB_LOCAL, STT_OBJECT),
 		 sec->idx, 0, sizeof(struct new_module));
 
   obj_string_patch(f, sec->idx, offsetof(struct new_module, name), m_name);
@@ -1242,9 +1248,11 @@ new_init_module(const char *m_name, struct obj_file *f, unsigned long m_size)
   struct obj_section *sec;
   void *image;
   int ret;
+  tgt_long m_addr;
 
   sec = obj_find_section(f, ".this");
   module = (struct new_module *)sec->contents;
+  m_addr = sec->header.sh_addr;
 
   module->size_of_struct = sizeof(*module);
   module->size = m_size;
@@ -1275,6 +1283,19 @@ new_init_module(const char *m_name, struct obj_file *f, unsigned long m_size)
       module->ex_table_end = sec->header.sh_addr + sec->header.sh_size;
     }
 
+  sec = obj_find_section(f, ".text.init");
+  if (sec)
+    {
+      module->runsize = sec->header.sh_addr - m_addr;
+    }
+  sec = obj_find_section(f, ".data.init");
+  if (sec)
+    {
+      if (!module->runsize || 
+          module->runsize > sec->header.sh_addr - m_addr)
+	module->runsize = sec->header.sh_addr - m_addr;
+    }
+
   if (!arch_init_module(f, module))
     return 0;
 
@@ -1520,8 +1541,9 @@ main(int argc, char **argv)
       : !old_create_mod_use_count(f))
     return 1;
 
-  if (!obj_allocate_commons_and_check_undefineds(f))
+  if (!obj_check_undefineds(f))
     return 1;
+  obj_allocate_commons(f);
 
   if (optind < argc)
     {
